#ifndef _DAG_H_
#define _DAG_H_

#include "list.h"
#include "global.h"

/* =================================================================
 *  Abstract data type for DAG.
 */
typedef struct {
    int  nInstructions;	   /* # of instructions; their names are 0,...,n-1 */
    char *blockID;	   /* unique identifier of the basic block	   */
    int  *instructionType; /* operation type of each instruction	   */
    int  *percentage;	   /* probability of an exit from this node        */
    double  *f1;	   /* frequency this exit is taken                 */
    double  *f2;	   /* frequency fall through at this exit          */
    ELEM **successors;	   /* adjacency lists of successors of nodes	   */
    int  *instructionIndex; /* Index of the instruction's mnemonic */
    int nDummyNodes; /* Number of dummy nodes needed for d > 1 instructions */
} DAG_type;

/*
 *  Maximum length of a line in the file that stores the DAG.
 */
#define MAX_LINE_LENGTH  2048

/* =================================================================
 *  Routines available for an instance of a DAG.
 */
DAG_type *getDAG( char *problem, BOOL doBreakEdges );
void	 printDAG( DAG_type *DAG );

#endif
