/********************************************************************
* helper.c                                                          *
*********************************************************************
* Written by Mike Chase                                             *
*********************************************************************
* Purpose: Generic code used in potentially many places; may also   *
* contain code that solves cross-platform issues, such as           *
* implementations of string manipulation routines common on Windows *
* but not ported to Unix.                                           *
********************************************************************/

#include <ctype.h>
#include <string.h>
#include "global.h"
#include "helper.h"

#ifndef WIN32

/*
 * Convert a string to lowercase
 */
char *
strlwr( char *string )
{
	char alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	int i;

	for( i = 0; i < strlen( string ); i++ ) {
		if( NULL != strchr( alphabet, string[i] ) ) {
			string[i] = tolower( string[i] );
		}
	}

	return string;
}
	
/*
 * Case-insensitive version of strcmp
 */
int
stricmp( const char *string1, const char *string2 )
{
	char s1[2049], s2[2049];

	strcpy( s1, string1 );
	strcpy( s2, string2 );

	strlwr( s1 );
	strlwr( s2 );

	return strcmp( s1, s2 );
}

#endif
