/********************************************************************
* instr.c                                                           *
*********************************************************************
* Written by Mike Chase                                             *
* Written on January 31, 2006                                       *
*********************************************************************
  Purpose: Tables of instruction information                        *
********************************************************************/

#include <string.h>
#include "instr.h"

// Get the index for a given instruction code
int getIndex(char *instr)
{
	int i = 0;

    for (i = 0; i < MAX_INSTRUCTIONS; i++)
	{
        if (strcmp(instr, instructionMap[i][0]) == 0)
			return i;
    }
	return 0;
}

// Return true if the instruction is an execution type; false otherwise
BOOL isExecution(int index)
{
	if (strcmp(instructionMap[index][1], "crand") == 0 ||
		strcmp(instructionMap[index][1], "cror") == 0 ||
		strcmp(instructionMap[index][1], "crnand") == 0 ||
		strcmp(instructionMap[index][1], "crnor") == 0 ||
		strcmp(instructionMap[index][1], "crxor") == 0 ||
		strcmp(instructionMap[index][1], "creqv") == 0 ||
		strcmp(instructionMap[index][1], "crandc") == 0 ||
		strcmp(instructionMap[index][1], "crorc") == 0 ||
		strcmp(instructionMap[index][1], "mcrf") == 0 ||
		strcmp(instructionMap[index][1], "stb") == 0 ||
		strcmp(instructionMap[index][1], "stbu") == 0 ||
		strcmp(instructionMap[index][1], "stbux") == 0 ||
		strcmp(instructionMap[index][1], "stbx") == 0 ||
		strcmp(instructionMap[index][1], "sth") == 0 ||
		strcmp(instructionMap[index][1], "sthu") == 0 ||
		strcmp(instructionMap[index][1], "sthux") == 0 ||
		strcmp(instructionMap[index][1], "sthx") == 0 ||
		strcmp(instructionMap[index][1], "stw") == 0 ||
		strcmp(instructionMap[index][1], "stwu") == 0 ||
		strcmp(instructionMap[index][1], "stwux") == 0 ||
		strcmp(instructionMap[index][1], "stwx") == 0 ||
		strcmp(instructionMap[index][1], "sthbrx") == 0 ||
		strcmp(instructionMap[index][1], "stwbrx") == 0 ||
		strcmp(instructionMap[index][1], "stfd") == 0 ||
		strcmp(instructionMap[index][1], "stfdu") == 0 ||
		strcmp(instructionMap[index][1], "stfdux") == 0 ||
		strcmp(instructionMap[index][1], "stfdx") == 0 ||
		strcmp(instructionMap[index][1], "stfs") == 0 ||
		strcmp(instructionMap[index][1], "stfsu") == 0 ||
		strcmp(instructionMap[index][1], "stfsux") == 0 ||
		strcmp(instructionMap[index][1], "stfsx") == 0 ||
		strcmp(instructionMap[index][1], "lwarx") == 0 ||
		strcmp(instructionMap[index][1], "stwcx") == 0 ||
		strcmp(instructionMap[index][1], "dcbf") == 0 ||
		strcmp(instructionMap[index][1], "dcbst") == 0 ||
		strcmp(instructionMap[index][1], "dcbi") == 0 ||
		strcmp(instructionMap[index][1], "dcbt") == 0 ||
		strcmp(instructionMap[index][1], "dcbtst") == 0 ||
		strcmp(instructionMap[index][1], "dcbz") == 0 ||
		strcmp(instructionMap[index][1], "adde") == 0 ||
		strcmp(instructionMap[index][1], "addeo") == 0 ||
		strcmp(instructionMap[index][1], "adde.") == 0 ||
		strcmp(instructionMap[index][1], "addeo.") == 0 ||
		strcmp(instructionMap[index][1], "subfe") == 0 ||
		strcmp(instructionMap[index][1], "subfeo") == 0 ||
		strcmp(instructionMap[index][1], "subfe.") == 0 ||
		strcmp(instructionMap[index][1], "subfeo.") == 0 ||
		strcmp(instructionMap[index][1], "addme") == 0 ||
		strcmp(instructionMap[index][1], "addmeo") == 0 ||
		strcmp(instructionMap[index][1], "addme.") == 0 ||
		strcmp(instructionMap[index][1], "addmeo.") == 0 ||
		strcmp(instructionMap[index][1], "subfme") == 0 ||
		strcmp(instructionMap[index][1], "subfmeo") == 0 ||
		strcmp(instructionMap[index][1], "subfme.") == 0 ||
		strcmp(instructionMap[index][1], "subfmeo.") == 0 ||
		strcmp(instructionMap[index][1], "addze") == 0 ||
		strcmp(instructionMap[index][1], "addzeo") == 0 ||
		strcmp(instructionMap[index][1], "addze.") == 0 ||
		strcmp(instructionMap[index][1], "addzeo.") == 0 ||
		strcmp(instructionMap[index][1], "subfze") == 0 ||
		strcmp(instructionMap[index][1], "subfzeo") == 0 ||
		strcmp(instructionMap[index][1], "subfze.") == 0 ||
		strcmp(instructionMap[index][1], "subfzeo.") == 0 ||
		strcmp(instructionMap[index][1], "mflr") == 0 ||
		strcmp(instructionMap[index][1], "mfctr") == 0 ||
		strcmp(instructionMap[index][1], "mtxer") == 0 ||
		strcmp(instructionMap[index][1], "mfxer") == 0 ||
		strcmp(instructionMap[index][1], "mtcrf") == 0 ||
		strcmp(instructionMap[index][1], "mcrxr") == 0 ||
		strcmp(instructionMap[index][1], "mfcr") == 0 ||
		strcmp(instructionMap[index][1], "stfiwx") == 0)
	{
		return TRUE;
	}

	return FALSE;

}

char *instructionMap[MAX_INSTRUCTIONS][3] = {
  { "ERROR",    "error",        "ERROR" },
  { "A",        "add",          "SFX" },
  { "ABSFL",    "fabs",         "FPU" },
  { "ADDC",     "addic",        "SFX" },
  { "ADDE",     "aze",          "SFX" },
  { "ADDE_R",   "aze.",         "SFX" },
  { "AFL",      "fadd",         "FPU" },
  { "AFS",      "fadds",        "FPU" },
  { "AI",       "addi",         "SFX" },
  { "AIU",      "addis",        "SFX" },
  { "AI_R",     "addic.",       "SFX" },
  { "ANDC",     "andc",         "SFX" },
  { "ANDC_R",   "andc.",        "SFX" },
  { "A_R",      "add.",         "SFX" },
  { "B",        "b",            "BPU" },
  { "BA",       "bclr",         "BPU" },
  { "BCF",      "bc",           "BPU" },
  { "BCT",      "bc",           "BPU" },
  { "BCTF",     "bc",           "BPU" },
  { "BCTT",     "bc",           "BPU" },
  { "BB_BEGIN", "dummy",        "DUMMY" },
  { "BF",       "bc",           "BPU" },
  { "BFBIT",    "bc",           "BPU" },
  { "BT",       "bc",           "BPU" },
  { "BTBIT",    "bc",           "BPU" },
  { "C4",       "cmpi",         "SFX" },
  { "CALL",     "bl",           "BPU" },
  { "CALLF",    "b",            "BPU" },
  { "CALLN",    "bl",           "BPU" },
  { "CALLNR",   "bl",           "BPU" },
  { "CFAA",     "addis",        "SFX" },
  { "CFL",      "fcmpu",        "FPU" },
  { "CFS",      "fcmpu",        "FPU" },
  { "CFSAA",    "addis",        "SFX" },
  { "CL4",      "cmpl",         "SFX" },
  { "CLA4",     "cmpli",        "SFX" },
  { "CNTLZ4",   "cntlzw",       "SFX" },
  { "COMP",     "neg",          "SFX" },
  { "COMPFL",   "fneg",         "FPU" },
  { "COMP_R",   "neg.",         "SFX" },
  { "CONSUME",  "dummy",        "DUMMY" },
  { "CRB_EQV",  "creqv",        "BPU" },
  { "CRB_N",    "crandc",       "BPU" },
  { "CRB_NNOT", "crandc",       "BPU" },
  { "CRB_NOR",  "crnor",        "BPU" },
  { "CRB_O",    "crorc",        "BPU" },
  { "CRB_ONOT", "crorc",        "BPU" },
  { "CR_EQV",   "creqv",        "BPU" },
  { "CR_N",     "crand",        "BPU" },
  { "CR_NAND",  "crnand",       "BPU" },
  { "CR_NNOT",  "crandc",       "BPU" },
  { "CR_NOR",   "crnor",        "BPU" },
  { "CR_O",     "cror",         "BPU" },
  { "CR_ONOT",  "crorc",        "BPU" },
  { "CR_X",     "crxor",        "BPU" },
  { "CVLS",     "frsp",         "FPU" },
  { "C_LR",     "instr",        "SFX+BPU" },
  { "C_LRFL",   "instr",        "FPU+BPU" },
  { "DCBZ",     "dcbz",         "LSU" },
  { "DFL",      "fdiv",         "FPU" },
  { "DFS",      "fdivs",        "FPU" },
  { "DIRCTIV",  "dummy",        "DUMMY" },
  { "DIVW",     "divw",         "CFX" },
  { "DIVWU",    "divwu",        "CFX" },
  { "EQV",      "eqv",          "SFX" },
  { "EXTS1",    "extsb",        "SFX" },
  { "EXTS1_R",  "extsb.",       "SFX" },
  { "EXTS2",    "extsh",        "SFX" },
  { "EXTS2_R",  "extsh.",       "SFX" },
  { "FCFID",    "fcfid",        "FPU" },
  { "FCTIDZ",   "fctidz",       "FPU" },
  { "FCTIWZ",   "fctiwz",       "FPU" },
  { "FMA",      "fmadd",        "FPU" },
  { "FMAS",     "fmadds",       "FPU" },
  { "FMS",      "fmsub",        "FPU" },
  { "FMSS",     "fmsubs",       "FPU" },
  { "FNMA",     "fnmadd",       "FPU" },
  { "FNMAS",    "fnmadds",      "FPU" },
  { "FNMS",     "fnmsub",       "FPU" },
  { "FNMSS",    "fnmsubs",      "FPU" },
  { "FSEL",     "fsel",         "FPU" },
  { "L1Z",      "lbz",          "LSU" },
  { "L1ZU",     "lbzu",         "LSU" },
  { "L2A",      "lha",          "LSU" },
  { "L2AU",     "lhau",         "LSU" },
  { "L2Z",      "lhzx",         "LSU" },
  { "L2ZU",     "lhzu",         "LSU" },
  { "L4A",      "lwz",          "LSU" },
  { "L4AU",     "lwzu",         "LSU" },
  { "L4Z",      "lwz",          "LSU" },
  { "L4ZU",     "lwzu",         "LSU" },
  { "LA",       "addi",         "SFX" },
  { "LBITCR",   "cr",           "BPU" },
  { "LCRI",     "cmp",          "SFX" },
  { "LCTR",     "mtspr",        "CFX" },
  { "LFCR",     "mfcr",         "CFX" },
  { "LFCTR",    "mfspr",        "CFX" },
  { "LFDU",     "lfdu",         "LSU" },
  { "LFL",      "lfd",          "LSU" },
  { "LFLR",     "mfspr",        "CFX" },
  { "LFS",      "lfs",          "LSU" },
  { "LFSU",     "lfsu",         "LSU" },
  { "LI",       "addi",         "SFX" },
  { "LIBIT",    "cr",           "BPU" },
  { "LIU",      "addis",        "SFX" },
  { "LLR",      "mtspr",        "CFX" },
  { "LR",       "ori",          "SFX" },
  { "LRBIT",    "cr",           "BPU" },
  { "LRCR",     "mcrf",         "BPU" },
  { "LRFL",     "fmr",          "FPU" },
  { "LRNOTBIT", "cr",           "BPU" },
  { "LR_R",     "or.",          "SFX" },
  { "LSI",      "lswi",         "LSU" },
  { "M",        "mullw",        "CFX" },
  { "MFL",      "fmul",         "FPU" },
  { "MFS",      "fmuls",        "FPU" },
  { "MTCRF",    "mtcrf",        "CFX" },
  { "MULHW",    "mulhw",        "CFX" },
  { "MULHWU",   "mulhwu",       "CFX" },
  { "M_R",      "mullw.",       "CFX" },
  { "N",        "and",          "SFX" },
  { "NABSFL",   "fnabs",        "FPU" },
  { "NAND",     "nand",         "SFX" },
  { "NILZ",     "andi.",        "SFX" },
  { "NIUZ",     "andis.",       "SFX" },
  { "NOP",      "ori",          "NOP" },
  { "NOR",      "nor",          "SFX" },
  { "N_R",      "and.",         "SFX" },
  { "O",        "or",           "SFX" },
  { "OIL",      "ori",          "SFX" },
  { "OIU",      "oris",         "SFX" },
  { "ORC",      "orc",          "SFX" },
  { "O_R",      "or.",          "SFX" },
  { "RI4",      "rlwimi",       "SFX" },
  { "RN4",      "rlwinm",       "SFX" },
  { "RN4_R",    "andi.",        "SFX" },
  { "S",        "subf",         "SFX" },
  { "SFI",      "subfic",       "SFX" },
  { "SFL",      "fsub",         "FPU" },
  { "SFS",      "fsubs",        "FPU" },
  { "SLL4",     "rlwinm",       "SFX" },
  { "SLL4_R",   "rlwinm.",      "SFX" },
  { "SQRT",     "fsqrt",        "FPU" },
  { "SQRTFS",   "fsqrts",       "FPU" },
  { "SRA4",     "srawi",        "SFX" },
  { "SRA4CA",   "srawi",        "SFX" },
  { "SRA4_R",   "srawi.",       "SFX" },
  { "SRL4",     "rlwinm",       "SFX" },
  { "SRL4_R",   "rlwinm.",      "SFX" },
  { "ST1A",     "stb",          "LSU" },
  { "ST1U",     "stbu",         "LSU" },
  { "ST1Z",     "stb",          "LSU" },
  { "ST2A",     "sth",          "LSU" },
  { "ST2U",     "sthu",         "LSU" },
  { "ST2Z",     "sth",          "LSU" },
  { "ST4A",     "stw",          "LSU" },
  { "ST4U",     "stwu",         "LSU" },
  { "ST4Z",     "stw",          "LSU" },
  { "STFDU",    "stfdu",        "LSU" },
  { "STFIWX",   "stfiwx",       "LSU" },
  { "STFL",     "stfd",         "LSU" },
  { "STFS",     "stfs",         "LSU" },
  { "STFSU",    "stfsu",        "LSU" },
  { "STSI",     "stswi",        "LSU" },
  { "SUBC",     "subfc",        "SFX" },
  { "SUBE",     "subfe",        "SFX" },
  { "SUBE_R",   "subfe",        "SFX" },
  { "S_R",      "subf.",        "SFX" },
  { "TCT4",     "twi",          "SFX" },
  { "TRAP",     "tw",           "SFX" },
  { "X",        "xor",          "SFX" },
  { "XIL",      "xori",         "SFX" },
  { "XIU",      "xoris",        "SFX" },
  { "X_R",      "xor.",         "SFX" }
};

int instructionTimes[MAX_INSTRUCTIONS][2] = {
	{  0,  0 },	{  1,  1 },	{  1,  3 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  3 },	{  1,  3 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  0,  0 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  3 },	{  1,  3 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  2 },	{  1,  1 },	{  1,  3 },	{  1,  1 }, {  0,  0 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  3 },	{  1,  1 },	{  1,  1 },	{  3,  3 },	{ 32, 32 },
	{ 18, 18 },	{  0,  0 },	{ 19, 20 },	{ 19, 20 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  3 },	{  1,  3 },	{  1,  3 },
	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },
	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  2 },	{  1,  2 },	{  1,  2 },
	{  1,  2 },	{  1,  2 },	{  1,  2 },	{  1,  2 },	{  1,  2 },	{  1,  2 },
	{  1,  2 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  3 },
	{  1,  1 },	{  1,  3 },	{  1,  3 },	{  1,  1 },	{  1,  3 },	{  1,  3 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  3 },	{  1,  1 },	{  1,  1 },	{  4,  4 },	{  3,  3 },
	{  1,  3 },	{  1,  3 },	{  1,  1 },	{  3,  3 },	{  3,  3 },	{  3,  3 },
	{  1,  1 },	{  1,  3 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  3 },	{  1,  3 },	{  1,  1 },	{  1,  1 },	{ 32, 32 },	{ 18, 18 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  3 },
	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },
	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },	{  1,  3 },
	{  1,  3 },	{  3,  3 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },	{  1,  1 },
};
