#ifndef _LS_H_
#define _LS_H_

#include "global.h"

/* =================================================================
 *  Abstract data type for list scheduler.
 */

typedef struct {
    char *blockID;	/* unique identifier of the basic block		   */
    int  n;		/* number of instructions; their names are 1,...,n */
    int  *varType;	/* operation type of each instruction i		   */
    int  *EST;		/* current earlist start time of i		   */
    int  *maxDelay;	/* critical path length from i to leaf node	   */
    int  *nPreds;	/* # of predecessors of a node not yet scheduled   */
    int  *nSuccs;	/* # of successors of a node			   */
    int  *costCoeffs;   /* cost coefficients for side exits                */
    double *costFreqs;	/* execution frequencies of the cost function	   */
    int  *dhasy;        /* the value for the dhasy heuristic               */
    int  *weight_est;   /* the value for the weighted estimate heuristic   */
    int  **branchDist;  /* distances to each of the side exits             */
    int  **cp;		/* critical path distances between all nodes       */
    int  **estimates;   /* estimated distances between all nodes           */
    BOOL *isSideExit;	/* is this instruction a side exit?		   */
    BOOL *isScheduled;	/* has this instruction been scheduled?		   */
    ELEM **latencySucc;
    ELEM **latencyPred;
    int  **pl;		/* path lengths between variables		   */
    int  *nDescs;	/* # of descendants of a node of each type	   */
    int  *distance;	/* estimate of distance to sink node		   */
    int  *eTime;	/* execution time of a node (max latency to succ)  */
    int  *sumLatencies;	/* sum of latencies to successors		   */
    //int  *nOps_s;	/* static number of operations of each type	   */
    //int  *nOps_d;	/* dynamic number of operations of each type	   */
} LS_type;

/* =================================================================
 *  Routine for scheduling a DAG using list scheduling.
 */
void listScheduler(
	DAG_type *DAG,
	int	 heuristic,
	int	 order[],
	int	 *finalCost,
	int 	 *finalLength,
	double	 *costWithFreq );

#endif
