
#include <stdio.h>
#include "global.h"
#include "processor.h"
#include "list.h"
#include "dag.h"
#include "csp.h"
#include "solve.h"
#include "settings.h"
#include "timer.h"
#include "ls.h"
#include "heuristic.h"

int
main( int argc, char *argv[] )
{
    int   lsCost = 0, optCost = 0;
    int   lsLength = 0; // number of cycles used by scheduler to schedule DAG
    int   optLength = 0;
    double   lsTime = 0.0,  optTime = 0.0;
    DAG_type *DAG;  // data dependencies DAG
    int      *schedule; // scheduling order of each instruction
    void     printSchedule( DAG_type *DAG, int schedule[] );
    int    foundOptimal, i;
    double costWithFreq;

    if( argc != 3 ) {
        fprintf( stderr, "Usage: schedule settings problem\n" );
        fprintf( stderr, "\tsettings -- file specifying experiment settings\n" );
        fprintf( stderr, "\tproblem  -- DAG to schedule\n" );
        exit( 6 );
    }

    /* Read the settings file */
    if( !readSettingsFile( argv[1] ) ) {
        exit( 6 );
    }

    /* Read in the DAG */
//printf("main: 1\n");
fflush( stdout );
    DAG = getDAG( argv[2], speculate() /* break all breakable edges if speculate returns TRUE*/ );
//printf("main: 2\n");
fflush( stdout );
    // printf( "-> DAG: %s\n", argv[2] );
printf( "%s", argv[2] );
    // Subtract 2 to account for dummy source and sink
    // printf( "-> DAG->n: %d\n", ( DAG->nInstructions - 2 ) );
printf( ":%d", DAG->nInstructions - 2 );
fflush( stdout );

    schedule = (int *)calloc( (size_t)DAG->nInstructions, sizeof(int) );
    for( i = 0; i < DAG->nInstructions; i++ ) {
        schedule[i] = -1;
    }

    if( shouldRunLS() ) {
        startGlobalStopwatch();
//printf("main: 3\n");
//fflush( stdout );
        listScheduler( DAG, getLSHeuristic(), schedule, &lsCost, &lsLength, &costWithFreq );
//printf("main: 4\n");
//fflush( stdout );
        lsTime = getGlobalStopwatchElapsedTime();
        if( shouldPrintSchedule() ) {
            printSchedule( DAG, schedule );
        }
        // printf( "-> LS Schedule cost: %d\n", lsCost );
printf( ":%d", lsCost );
printf( ":%0.0f", costWithFreq );
fflush( stdout );
//printf("\n");
//exit(6);
        // printf( "-> LS Elapsed time: %fs\n", lsTime );

        for( i = 0; i < DAG->nInstructions; i++ ) {
            schedule[i] = -1;
        }
    }

    if( shouldRunOptimal() ) {
        startGlobalStopwatch();
        optimalScheduler( DAG, schedule, &foundOptimal, &optCost, &optLength, &costWithFreq );
        optTime = getGlobalStopwatchElapsedTime();

printf( ":%d", optCost );
printf( ":%0.0f", costWithFreq );
printf( ":%0.2f", optTime );
if( foundOptimal ) printf( ":opt\n" );
else               printf( ":not\n" );
fflush( stdout );

        if( foundOptimal ) {
            if( shouldPrintSchedule() ) {
                printSchedule( DAG, schedule );
            }
            //printf( "-> OPT Schedule cost: %d\n", optCost );
            //printf( "-> OPT Elapsed time: %fs\n", optTime );
        }
        else {
            // printf( "-> OPT timed out\n" );
            if( shouldPrintSchedule() ) {
                printSchedule( DAG, schedule );
            }
            // printf( "-> OPT partial cost: %d\n", optCost );
            // printf( "-> OPT time: %fs\n", optTime );
        }
    }

    if( shouldRunLS() && shouldRunOptimal() ) {
        // printf( "-> Cost difference: %d\n", ( lsCost - optCost ) );
        // printf( "-> Time difference: %fs\n", ( max(optTime, 0.0f) - lsTime ) );

        if( lsCost < optCost ) {
            printf( "Error: List scheduler produced a shorter schedule than optimal scheduler\n" );
            printf( "Block length is %d\n", DAG->nInstructions );
fflush( stdout );
        }
    }

    return( 0 );
}

void
printSchedule( DAG_type *DAG, int schedule[] )
{
    int  time, i, m;

    m = schedule[DAG->nInstructions-1];
    for( time = 1; time <= m; time++ ) {
    printf( "Instruction(s) at time %d:", time );
    for( i = 0; i < DAG->nInstructions; i++ ) {
        if( schedule[i] == time ) {
        printf( " %d(%d)", i, DAG->instructionType[i] );
        }
    }
    printf( "\n" );
    }
}

