#ifndef _PROCESSOR_H_
#define _PROCESSOR_H_

/* =================================================================
 *  Defining the processor
 */

/*
 *  Number of different instruction types.
 *  Assumption: there is at least one functional unit for each type of
 *  instruction, each instruction can only execute on its corresponding
 *  functional unit, only one functional unit is involved in the
 *  execution of an instruction.
 */
#define   N_INSTRUCTION_TYPES	7

#define   INTEGER_INSTR    0
#define   FLOAT_INSTR      1
#define   LDST_INSTR       2
#define   BRANCH_INSTR     3
#define   SIMPLEINT_INSTR  4
#define   COMPLEXINT_INSTR 5
#define   PSEUDO_INSTR     6


/*
 * Special field to indicate that all (or all remaining) instruction
 * types can be selected uniformly when list scheduling
 */
#define   INSTR_DONT_CARE 7


/*
 * Functional Unit Types
 */
#define	  N_FUNCTIONAL_UNIT_TYPES 7

#define	  INT_UNIT    0
#define	  FLOAT_UNIT  1
#define	  LDST_UNIT   2
#define	  BRANCH_UNIT 3
#define   SFX_UNIT    4
#define   CFX_UNIT    5
#define	  PSEUDO_UNIT 6

/* =================================================================
 *  Routines for answering queries about the processor model
 */
int  getInstructionType( char *line );
int  nFunctionalUnits( int type );
void setNumFunctionalUnits( int type, int nUnits );
void setIssueWidth( int width );
int  getIssueWidth();
void setMaxLatency( int latency );
int  getMaxLatency();
int  getInstructionUnitMapping( int instructionType );
void setInstructionUnitMapping( int instructionType, int unitType );
int  getInstructionTypePreference( int rank );
void setInstructionTypePreference( int rank, int instructionType );
int getMaxUnitsOfAnyType();

#endif
